import reg
import sys
import signal


def parse_transform(string: str, automaton_type: str) -> reg.DFA:

    try:
        parser = reg.Parser()
        if automaton_type == "DFA":
            automaton = parser.str_to_dfa(string)
        elif automaton_type == "NFA":
            automaton = parser.str_to_nfa(string).determinize()

        return automaton

    except reg.Parser.ParsingError as message:
        print("Parsing error:", message)
        exit(1)
    except AttributeError as message:
        print("Parsing error:", message)
        exit(1)


def check_task(dfa: reg.DFA, task: str) -> bool:

    if task == "TOT" and not dfa.is_total():
        print("DFA není totální.")
        return False

    if task == "MIN" and not dfa.is_minimal():
        print("DFA není minimální.")
        return False

    canonical = dfa.is_canonical()
    if task == "CAN" and not canonical:
        print("DFA není kanonický.")
        return False

    if task == "TOC" and (not dfa.is_total() or not canonical):
        if not dfa.is_total():
            print("DFA není totální.")
        if not canonical:
            print("DFA není kanonický.")
        return False

    if task == "MIC" and (not dfa.is_minimal() or not canonical):
        if not dfa.is_minimal():
            print("DFA není minimální.")
        if not canonical:
            print("DFA není kanonický.")
        return False

    #elif solution_type == "EFA":
    #    parser = Parser()
    #    efa = parser.str_to_efa(solution)
    #    return efa.eliminate_epsilon.determinize()
    return True

def main():
    signal.alarm(60)

    with open(sys.argv[1]) as student_file:
        student_solution = student_file.read()

    for argument in sys.argv:
        if argument[:2] == "-o":
            teacher_solution = argument[2:]

    try:
        task_prefix, teacher_solution = teacher_solution.split(":", 1)
        assert len(task_prefix) >= 7, "Teachers solution could not be parsed correctly."
        automata_type, task, _ = task_prefix.split("-", 2)
        assert len(automata_type) == 3 and len(task) == 3, "Teachers solution could not be parsed correctly."

        student_dfa = parse_transform(student_solution, automata_type)
        teacher_dfa = parse_transform(teacher_solution, automata_type)

        #
        if teacher_dfa.is_empty() and student_dfa.is_empty():
            print("Správné řešení.")
            exit(0)

        elif student_dfa.characters != teacher_dfa.characters:
            print("Abecedy zadaného jazyka a studentova řešení se liší "
                  "(abeceda je odvozena ze všech písmen objevujících se v přechodové funkci).")
            print("Nesprávné řešení.")
            exit(1)

        result = reg.DFA.is_equivalent(student_dfa, teacher_dfa)
        if check_task(student_dfa, task) and result:
            print("Správné řešení.")
            exit(0)

        if not result:
            student_word, teacher_word = result.left_counterexample, result.right_counterexample

            if student_word is None:
                print("Studentovo řešení je podmnožinou zadaného jazyka.")
            else:
                print("Příklad slova, které je ve studentově řešení a není v zadaném jazyce: ", student_word)

            if teacher_word is None:
                print("Studentovo řešení je nadmnožinou zadaného jazyka.")
            else:
                print("Příklad slova, které chybí ve studentově řešení a je v zadaném jazyce: ", teacher_word)

        print("Nesprávné řešení.")
        exit(1)

    except Exception as ex:
        print("Error inside of fja_checker:", ex.args)
        exit(1)

main()